from adafruit_crickit import crickit

class Servo:
    Servos = [crickit.servo_1, crickit.servo_2,
              crickit.servo_3, crickit.servo_4]
    def __init__(self, ID):
        '''
        Initialize a Servo

        :param ID: ID of the Servo [0,1,2,3]
        :type ID: int
        '''
        self.servo = self.Servos[ID]

    def angle(self, degree):
        '''
        Set the angle to rotate to

        :param degree: degree of the Servo
        :type degree: int
        '''
        self.servo.angle = degree


class ContiniousServo:
    ContiniousServos = [crickit.continuous_servo_1, crickit.continuous_servo_2,
                        crickit.continuous_servo_3, crickit.continuous_servo_4]
    def __init__(self, ID):
        '''
        Initialize a Continious Servo

        :param ID: ID of the Continious Servo [0,1,2,3]
        :type ID: int
        '''
        self.servo = self.ContiniousServos[ID]

    def throttle(self, power):
        '''
        Set the throttle of the Continious Servo

        :param power: Power of the Continious Servo -1 to 1
        :type power: float
        '''
        self.servo.throttle = power



import pygame

class Joystick:
    def __init__(self, ID, deadband=0):
        '''
        Setup Joystick control using pygame
        '''
        pygame.init()
        pygame.joystick.init()
        self.deadband = abs(deadband)
        self.newJoystick(ID)

    def info(self):
        '''
        Print out info regarding all connected joystics
        '''
        jstickCount = pygame.joystick.get_count()
        print("Number of Joysticks connected: \n", jstickCount)
        # For each joystick:
        for i in range(jstickCount):
            joystick = pygame.joystick.Joystick(i)
            joystick.init()

            print("Joystick {}".format(i))
            # Get the name from the OS for the controller/joystick.
            print("Joystick name: {}".format(joystick.get_name()))
            # Usually axis run in pairs, up/down for one, and left/right
            print("Number of axes: {}".format(joystick.get_numaxes()))
            print("Number of buttons: {}".format(joystick.get_numbuttons()))

    def newJoystick(self, deviceID, _return=False):
        '''
        Setup a new joystick

        :param deviceID: ID of the joystick
        :type deviceID: int
        :param _return: return joystick instance, defaults to False
        :type _return: bool, optional
        :return: joystick instance if _return == True
        :rtype: pygame.joystick.Joystick
        '''
        self.jstick = pygame.joystick.Joystick(deviceID)
        self.jstick.init()
        if _return:
            return self.jstick

    def getAxis(self, axisID):
        '''
        Get the value of the axis

        :param axisID: Id of the axis
        :type axisID: int
        :return: value of the axis
        :rtype: float
        '''
        pygame.event.pump()
        _return = self.jstick.get_axis(axisID) if self.jstick.get_axis(axisID) > self.deadband or self.jstick.get_axis(axisID) < -self.deadband else 0
        return _return

    def getButton(self, buttonID):
        '''
        Get the state of the button

        :param buttonID: Id of the button
        :type buttonID: int
        :return: state of the button
        :rtype: bool
        '''
        return self.jstick.get_button(buttonID)



from adafruit_crickit import crickit

class Motor:
    Motors = [crickit.dc_motor_1, crickit.dc_motor_2]
    def __init__(self, ID):
        '''
        Inintialize the DC Motor

        :param ID: The ID of the Motor [0,1]
        :type ID: int
        '''
        self.motor = self.Motors[ID]

    def throttle(self, power):
        '''
        Input power for the Motor

        :param power: Value from -1 to 1
        :type power: float
        '''
        self.motor.throttle = power



# Run the following command to install package:
# pip install --user .
from setuptools import setup

def readme():
    try:
      with open('README.rst') as f:
          return f.read()
    except:
      return "Minibot"

setup(name='minilib',
      version='0.22',
      long_description=readme(),
      description='Package for adafruit powered servo bots',
      url='http://github.com/atikul99/minibot',
      author='Abir',
      author_email='atikul10152002@gmail.com',
      license='MIT',
      install_requires=[
          'pygame', 'adafruit-circuitpython-crickit'
      ],
      packages=['minilib'],
      zip_safe=False,
      classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.7',
  ],
)



class ArcadeDrive:
    def __init__(self, left, right, a=0, b=1):
        '''
        Setup Arcade drive

        :param left: left side drive
        :type right: continuous_servo
        :param rightServo: right side drive
        :type rightServo: continuous_servo
        '''
        self.left = left
        self.right = right
        self.a = a
        self.b = b

    def calculate(self, fwd, rcw, a, b):
        '''
        Arcade algorithm to compute left and right wheel commands
        from forward and rotate-clockwise joystick commands.

        a=0 and b=1 provides the WPILib arcade behavior.

        :param fwd: Forward command (-1 to +1)
        :type fwd: float
        :param turn: Rotate command (-1 to +1)
        :type turn: float
        :param a: amount to turn at 100% fwd in the range 0 to 1, defaults to 0
        :type a: int, optional
        :param b: amount to turn at   0% fwd in the range 0 to 1, defaults to 1
        :type b: int, optional
        :return: leftPower, rightPower
        :rtype: (float, float)
        '''
        def L(fwd, turn, a, b): return fwd + b*turn*(1-fwd)
        def R(fwd, rcw, a, b): return fwd - b*rcw + fwd*rcw*(b-a-1)
        if fwd >= 0:
            if rcw >= 0:
                left = L(fwd, rcw, a, b)
                right = R(fwd, rcw, a, b)
            else:
                left = R(fwd, -rcw, a, b)
                right = L(fwd, -rcw, a, b)
        else:
            if rcw >= 0:
                left = -R(-fwd, rcw, a, b)
                right = -L(-fwd, rcw, a, b)
            else:
                left = -L(-fwd, -rcw, a, b)
                right = -R(-fwd, -rcw, a, b)
        return left, right

    def drive(self, forwardPower, steerPower):
        '''
        Drive given the forward and steer axis power
        Meant to be run inside a while loop

        :param forwardPower: Forward power from joystick axis
        :type forwardPower: float
        :param steerPower: Steer power from joystick axis
        :type steerPower: float
        '''
        self.leftPower, self.rightPower = self.calculate(
            forwardPower, steerPower, self.a, self.b)
        self.left.throttle(self.leftPower)
        self.right.throttle(self.rightPower)


class TankDrive:
    def __init__(self, left, right):
        '''
        Setup Tank drive

        :param LeftServo: Servo for the left side drive
        :type LeftServo: continuous_servo
        :param rightServo: Servo for the right side drive
        :type rightServo: continuous_servo
        '''
        self.left = left
        self.right = right

    def drive(self, leftPower, rightPower):
        '''
        Drive given the left and right axis power
        Meant to be run inside a while loop

        :param leftPower: Forward power from joystick axis
        :type leftPower: float
        :param rightPower: Steer power from joystick axis
        :type rightPower: float
        '''
        self.left.throttle(leftPower)
        self.right.throttle(rightPower)
